@@ -28,9 +28,6 @@ from compas.geometry.transformations import matrix_inverse
 from compas.geometry.transformations import translation_from_matrix
 
 
-__all__ = ['Transformation']
-
-
 class Transformation(Data):
     """The ``Transformation`` represents a 4x4 transformation matrix.
 
@@ -59,6 +56,16 @@ class Transformation(Data):
     >>> Tinv = T.inverse()
     """
 
+    @property
+    def DATASCHEMA(self):
+        from schema import Schema
+        from compas.data import is_float4x4
+        return Schema({"matrix": is_float4x4})
+
+    @property
+    def JSONSCHEMANAME(self):
+        return 'transformation'
+
     def __init__(self, matrix=None):
         """Construct a transformation from a 4x4 transformation matrix.
         """
@@ -125,37 +132,6 @@ class Transformation(Data):
             self.matrix[3][:]]
         return cls(matrix)
 
-    @property
-    def DATASCHEMA(self):
-        from schema import Schema
-        from compas.data import is_float4x4
-        return Schema({"matrix": is_float4x4})
-
-    @property
-    def JSONSCHEMA(self):
-        from compas import versionstring
-        schema = {
-            "$schema": "http://json-schema.org/draft-07/schema#",
-            "$id": "https://github.com/compas-dev/compas/schemas/transformation.json",
-            "$compas": versionstring,
-            "type": "object",
-            "properties": {
-                "matrix": {
-                    "type": "array",
-                    "minItems": 4,
-                    "maxItems": 4,
-                    "items": {
-                        "type": "array",
-                        "minItems": 4,
-                        "maxItems": 4,
-                        "items": {"type": "number"}
-                    }
-                }
-            },
-            "required": ["matrix"]
-        }
-        return schema
-
     @property
     def data(self):
         """Return a ``Transformation`` object's to a data dict.
