@@ -3,9 +3,6 @@ from __future__ import absolute_import
 from __future__ import division
 
 from functools import partial
-import compas_rhino
-
-from compas_rhino.artists._artist import BaseArtist
 
 from compas.utilities import color_to_colordict
 from compas.utilities import pairwise
@@ -14,14 +11,14 @@ from compas.geometry import scale_vector
 from compas.geometry import centroid_polygon
 from compas.geometry import centroid_points
 
+import compas_rhino
+from compas.artists import MeshArtist
+from .artist import RhinoArtist
 
 colordict = partial(color_to_colordict, colorformat='rgb', normalize=False)
 
 
-__all__ = ['MeshArtist']
-
-
-class MeshArtist(BaseArtist):
+class MeshArtist(RhinoArtist, MeshArtist):
     """Artists for drawing mesh data structures.
 
     Parameters
@@ -30,112 +27,134 @@ class MeshArtist(BaseArtist):
         A COMPAS mesh.
     layer : str, optional
         The name of the layer that will contain the mesh.
-
-    Attributes
-    ----------
-    mesh : :class:`compas.datastructures.Mesh`
-        The COMPAS mesh associated with the artist.
-    layer : str
-        The layer in which the mesh should be contained.
-    color_vertices : 3-tuple
-        Default color of the vertices.
-    color_edges : 3-tuple
-        Default color of the edges.
-    color_faces : 3-tuple
-        Default color of the faces.
-
-    Examples
-    --------
-    .. code-block:: python
-
-        import compas
-        from compas.datastructures import Mesh
-        from compas_rhino.artists import MeshArtist
-
-        mesh = Mesh.from_obj(compas.get('faces.obj'))
-
-        artist = MeshArtist(mesh, layer='COMPAS::MeshArtist')
-        artist.clear_layer()
-        artist.draw_faces(join_faces=True)
-        artist.draw_vertices(color={key: '#ff0000' for key in mesh.vertices_on_boundary()})
-        artist.draw_edges()
-        artist.redraw()
-
     """
 
-    def __init__(self, mesh, layer=None):
-        super(MeshArtist, self).__init__()
-        self._mesh = None
-        self._vertex_xyz = None
-        self.mesh = mesh
-        self.layer = layer
-        self.color_vertices = (255, 255, 255)
-        self.color_edges = (0, 0, 0)
-        self.color_faces = (0, 0, 0)
+    def __init__(self,
+                 mesh,
+                 layer=None,
+                 vertices=None,
+                 edges=None,
+                 faces=None,
+                 vertexcolor=(255, 255, 255),
+                 edgecolor=(0, 0, 0),
+                 facecolor=(221, 221, 221),
+                 show_mesh=False,
+                 show_vertices=True,
+                 show_edges=True,
+                 show_faces=True,
+                 **kwargs):
 
-    @property
-    def mesh(self):
-        return self._mesh
+        super(MeshArtist, self).__init__(mesh=mesh, layer=layer, **kwargs)
 
-    @mesh.setter
-    def mesh(self, mesh):
-        self._mesh = mesh
-        self._vertex_xyz = None
-
-    @property
-    def vertex_xyz(self):
-        """dict:
-        The view coordinates of the mesh vertices.
-        The view coordinates default to the actual mesh coordinates.
-        """
-        if not self._vertex_xyz:
-            return {vertex: self.mesh.vertex_attributes(vertex, 'xyz') for vertex in self.mesh.vertices()}
-        return self._vertex_xyz
-
-    @vertex_xyz.setter
-    def vertex_xyz(self, vertex_xyz):
-        self._vertex_xyz = vertex_xyz
+        self.vertices = vertices
+        self.edges = edges
+        self.faces = faces
+        self.vertex_color = vertexcolor
+        self.edge_color = edgecolor
+        self.face_color = facecolor
+        self.show_mesh = show_mesh
+        self.show_vertices = show_vertices
+        self.show_edges = show_edges
+        self.show_faces = show_faces
 
     # ==========================================================================
     # clear
     # ==========================================================================
 
-    def clear_by_name(self):
-        """Clear all objects in the "namespace" of the associated mesh."""
+    def clear(self):
         guids = compas_rhino.get_objects(name="{}.*".format(self.mesh.name))
         compas_rhino.delete_objects(guids, purge=True)
 
-    def clear_layer(self):
-        """Clear the main layer of the artist."""
-        if self.layer:
-            compas_rhino.clear_layer(self.layer)
+    def clear_mesh(self):
+        guids = compas_rhino.get_objects(name="{}.mesh".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_vertices(self):
+        guids = compas_rhino.get_objects(name="{}.vertex.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_edges(self):
+        guids = compas_rhino.get_objects(name="{}.edge.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_faces(self):
+        guids = compas_rhino.get_objects(name="{}.face.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_vertexnormals(self):
+        guids = compas_rhino.get_objects(name="{}.vertexnormal.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_facenormals(self):
+        guids = compas_rhino.get_objects(name="{}.facenormal.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_vertexlabels(self):
+        guids = compas_rhino.get_objects(name="{}.vertexlabel.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_edgelabels(self):
+        guids = compas_rhino.get_objects(name="{}.edgelabel.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
+
+    def clear_facelabels(self):
+        guids = compas_rhino.get_objects(name="{}.facelabel.*".format(self.mesh.name))
+        compas_rhino.delete_objects(guids, purge=True)
 
     # ==========================================================================
     # draw
     # ==========================================================================
 
-    def draw(self):
-        """Draw the mesh using the chosen visualisation settings.
+    def draw(self, vertices=None, edges=None, faces=None, vertexcolor=None, edgecolor=None, facecolor=None, join_faces=False):
+        """Draw the mesh using the chosen visualization settings.
+
+        Parameters
+        ----------
+        vertices : list, optional
+            A list of vertices to draw.
+            Default is ``None``, in which case all vertices are drawn.
+        edges : list, optional
+            A list of edges to draw.
+            The default is ``None``, in which case all edges are drawn.
+        faces : list, optional
+            A selection of faces to draw.
+            The default is ``None``, in which case all faces are drawn.
+        vertexcolor : tuple or dict of tuple, optional
+            The color specification for the vertices.
+            The default color is the value of ``~MeshArtist.default_vertexcolor``.
+        edgecolor : tuple or dict of tuple, optional
+            The color specification for the edges.
+            The default color is the value of ``~MeshArtist.default_edgecolor``.
+        facecolor : tuple or dict of tuple, optional
+            The color specification for the faces.
+            The default color is the value of ``~MeshArtist.default_facecolor``.
+        join_faces : bool, optional
+            Join the faces into 1 mesh.
+            Default is ``False``, in which case the faces are drawn as individual meshes.
 
         Returns
         -------
-        list
-            The GUIDs of the created Rhino objects.
+        None
 
         """
-        guids = self.draw_vertices()
-        guids += self.draw_faces()
-        guids += self.draw_edges()
-        return guids
+        self.clear()
+        if self.show_mesh:
+            self.draw_mesh()
+        if self.show_vertices:
+            self.draw_vertices(vertices=vertices, color=vertexcolor)
+        if self.show_edges:
+            self.draw_edges(edges=edges, color=edgecolor)
+        if self.show_faces:
+            self.draw_faces(faces=faces, color=facecolor, join_faces=join_faces)
 
-    def draw_mesh(self, color=(0, 0, 0), disjoint=False):
+    def draw_mesh(self, color=None, disjoint=False):
         """Draw the mesh as a consolidated RhinoMesh.
 
         Parameters
         ----------
         color : tuple, optional
             The color of the mesh.
-            Default is black, ``(0, 0, 0)``.
+            Default is the value of ``~MeshArtist.default_color``.
         disjoint : bool, optional
             Draw the faces of the mesh with disjoint vertices.
             Default is ``False``.
@@ -150,7 +169,8 @@ class MeshArtist(BaseArtist):
         The mesh should be a valid Rhino Mesh object, which means it should have only triangular or quadrilateral faces.
         Faces with more than 4 vertices will be triangulated on-the-fly.
         """
-        vertex_index = self.mesh.key_index()
+        color = color or self.default_color
+        vertex_index = self.mesh.vertex_index()
         vertex_xyz = self.vertex_xyz
         vertices = [vertex_xyz[vertex] for vertex in self.mesh.vertices()]
         faces = [[vertex_index[vertex] for vertex in self.mesh.face_vertices(face)] for face in self.mesh.faces()]
@@ -166,10 +186,8 @@ class MeshArtist(BaseArtist):
                 vertices.append(centroid_polygon([vertices[index] for index in face]))
                 for a, b in pairwise(face + face[0:1]):
                     new_faces.append([centroid, a, b, b])
-            else:
-                continue
         layer = self.layer
-        name = "{}".format(self.mesh.name)
+        name = "{}.mesh".format(self.mesh.name)
         guid = compas_rhino.draw_mesh(vertices, new_faces, layer=layer, name=name, color=color, disjoint=disjoint)
         return [guid]
 
@@ -178,12 +196,12 @@ class MeshArtist(BaseArtist):
 
         Parameters
         ----------
-        vertices : list
+        vertices : list, optional
             A selection of vertices to draw.
             Default is ``None``, in which case all vertices are drawn.
         color : tuple or dict of tuple, optional
-            The color specififcation for the vertices.
-            The default is white, ``(255, 255, 255)``.
+            The color specification for the vertices.
+            The default is the value of ``~MeshArtist.default_vertexcolor``.
 
         Returns
         -------
@@ -191,56 +209,18 @@ class MeshArtist(BaseArtist):
             The GUIDs of the created Rhino objects.
 
         """
-        vertices = vertices or list(self.mesh.vertices())
+        self.vertex_color = color
+        vertices = vertices or self.vertices
         vertex_xyz = self.vertex_xyz
-        vertex_color = colordict(color, vertices, default=self.color_vertices)
         points = []
         for vertex in vertices:
             points.append({
                 'pos': vertex_xyz[vertex],
                 'name': "{}.vertex.{}".format(self.mesh.name, vertex),
-                'color': vertex_color[vertex]})
-        return compas_rhino.draw_points(points, layer=self.layer, clear=False, redraw=False)
-
-    def draw_faces(self, faces=None, color=None, join_faces=False):
-        """Draw a selection of faces.
-
-        Parameters
-        ----------
-        faces : list, optional
-            A selection of faces to draw.
-            The default is ``None``, in which case all faces are drawn.
-        color : tuple or dict of tuple, optional
-            The color specififcation for the faces.
-            The default color is black ``(0, 0, 0)``.
-        join_faces : bool, optional
-            Join the faces into 1 mesh.
-            Default is ``False``, in which case the faces are drawn as individual meshes.
-
-        Returns
-        -------
-        list
-            The GUIDs of the created Rhino objects.
-
-        """
-        faces = faces or list(self.mesh.faces())
-        vertex_xyz = self.vertex_xyz
-        face_color = colordict(color, faces, default=self.color_faces)
-        facets = []
-        for face in faces:
-            facets.append({
-                'points': [vertex_xyz[vertex] for vertex in self.mesh.face_vertices(face)],
-                'name': "{}.face.{}".format(self.mesh.name, face),
-                'color': face_color[face]})
-        guids = compas_rhino.draw_faces(facets, layer=self.layer, clear=False, redraw=False)
-        if not join_faces:
-            return guids
-        guid = compas_rhino.rs.JoinMeshes(guids, delete_input=True)
-        compas_rhino.rs.ObjectLayer(guid, self.layer)
-        compas_rhino.rs.ObjectName(guid, '{}'.format(self.mesh.name))
-        if color:
-            compas_rhino.rs.ObjectColor(guid, color)
-        return [guid]
+                'color': self.vertex_color.get(vertex, self.default_vertexcolor)
+            })
+        guids = compas_rhino.draw_points(points, layer=self.layer, clear=False, redraw=False)
+        return guids
 
     def draw_edges(self, edges=None, color=None):
         """Draw a selection of edges.
@@ -251,8 +231,8 @@ class MeshArtist(BaseArtist):
             A selection of edges to draw.
             The default is ``None``, in which case all edges are drawn.
         color : tuple or dict of tuple, optional
-            The color specififcation for the edges.
-            The default color is black, ``(0, 0, 0)``.
+            The color specification for the edges.
+            The default color is the value of ``~MeshArtist.default_edgecolor``.
 
         Returns
         -------
@@ -260,17 +240,59 @@ class MeshArtist(BaseArtist):
             The GUIDs of the created Rhino objects.
 
         """
-        edges = edges or list(self.mesh.edges())
+        self.edge_color = color
+        edges = edges or self.edges
         vertex_xyz = self.vertex_xyz
-        edge_color = colordict(color, edges, default=self.color_edges)
         lines = []
         for edge in edges:
             lines.append({
                 'start': vertex_xyz[edge[0]],
                 'end': vertex_xyz[edge[1]],
-                'color': edge_color[edge],
-                'name': "{}.edge.{}-{}".format(self.mesh.name, *edge)})
-        return compas_rhino.draw_lines(lines, layer=self.layer, clear=False, redraw=False)
+                'color': self.edge_color.get(edge, self.default_edgecolor),
+                'name': "{}.edge.{}-{}".format(self.mesh.name, *edge)
+            })
+        guids = compas_rhino.draw_lines(lines, layer=self.layer, clear=False, redraw=False)
+        return guids
+
+    def draw_faces(self, faces=None, color=None, join_faces=False):
+        """Draw a selection of faces.
+
+        Parameters
+        ----------
+        faces : list, optional
+            A selection of faces to draw.
+            The default is ``None``, in which case all faces are drawn.
+        color : tuple or dict of tuple, optional
+            The color specification for the faces.
+            The default color is the value of ``~MeshArtist.default_facecolor``.
+        join_faces : bool, optional
+            Join the faces into 1 mesh.
+            Default is ``False``, in which case the faces are drawn as individual meshes.
+
+        Returns
+        -------
+        list
+            The GUIDs of the created Rhino objects.
+
+        """
+        self.face_color = color
+        faces = faces or self.faces
+        vertex_xyz = self.vertex_xyz
+        facets = []
+        for face in faces:
+            facets.append({
+                'points': [vertex_xyz[vertex] for vertex in self.mesh.face_vertices(face)],
+                'name': "{}.face.{}".format(self.mesh.name, face),
+                'color': self.face_color.get(face, self.default_facecolor)
+            })
+        guids = compas_rhino.draw_faces(facets, layer=self.layer, clear=False, redraw=False)
+        if join_faces:
+            guid = compas_rhino.rs.JoinMeshes(guids, delete_input=True)
+            compas_rhino.rs.ObjectLayer(guid, self.layer)
+            compas_rhino.rs.ObjectName(guid, '{}.mesh'.format(self.mesh.name))
+            compas_rhino.rs.ObjectColor(guid, color)
+            guids = [guid]
+        return guids
 
     # ==========================================================================
     # draw normals
@@ -298,7 +320,7 @@ class MeshArtist(BaseArtist):
 
         """
         vertex_xyz = self.vertex_xyz
-        vertices = vertices or list(self.mesh.vertices())
+        vertices = vertices or self.vertices
         lines = []
         for vertex in vertices:
             a = vertex_xyz[vertex]
@@ -309,7 +331,8 @@ class MeshArtist(BaseArtist):
                 'end': b,
                 'color': color,
                 'name': "{}.vertexnormal.{}".format(self.mesh.name, vertex),
-                'arrow': 'end'})
+                'arrow': 'end'
+            })
         return compas_rhino.draw_lines(lines, layer=self.layer, clear=False, redraw=False)
 
     def draw_facenormals(self, faces=None, color=(0, 255, 255), scale=1.0):
@@ -334,7 +357,7 @@ class MeshArtist(BaseArtist):
 
         """
         vertex_xyz = self.vertex_xyz
-        faces = faces or list(self.mesh.faces())
+        faces = faces or self.faces
         lines = []
         for face in faces:
             a = centroid_points([vertex_xyz[vertex] for vertex in self.mesh.face_vertices(face)])
@@ -345,7 +368,8 @@ class MeshArtist(BaseArtist):
                 'end': b,
                 'name': "{}.facenormal.{}".format(self.mesh.name, face),
                 'color': color,
-                'arrow': 'end'})
+                'arrow': 'end'
+            })
         return compas_rhino.draw_lines(lines, layer=self.layer, clear=False, redraw=False)
 
     # ==========================================================================
@@ -361,7 +385,7 @@ class MeshArtist(BaseArtist):
             A dictionary of vertex labels as vertex-text pairs.
             The default value is ``None``, in which case every vertex will be labelled with its key.
         color : tuple or dict of tuple, optional
-            The color sepcification of the labels.
+            The color specification of the labels.
             The default color is the same as the default vertex color.
 
         Returns
@@ -371,59 +395,23 @@ class MeshArtist(BaseArtist):
 
         """
         if not text or text == 'key':
-            vertex_text = {vertex: str(vertex) for vertex in self.mesh.vertices()}
+            vertex_text = {vertex: str(vertex) for vertex in self.vertices}
         elif text == 'index':
-            vertex_text = {vertex: str(index) for index, vertex in enumerate(self.mesh.vertices())}
+            vertex_text = {vertex: str(index) for index, vertex in enumerate(self.vertices)}
         elif isinstance(text, dict):
             vertex_text = text
         else:
             raise NotImplementedError
         vertex_xyz = self.vertex_xyz
-        vertex_color = colordict(color, vertex_text.keys(), default=self.color_vertices)
+        vertex_color = colordict(color, vertex_text.keys(), default=self.default_vertexcolor)
         labels = []
         for vertex in vertex_text:
             labels.append({
                 'pos': vertex_xyz[vertex],
                 'name': "{}.vertexlabel.{}".format(self.mesh.name, vertex),
                 'color': vertex_color[vertex],
-                'text': vertex_text[vertex]})
-        return compas_rhino.draw_labels(labels, layer=self.layer, clear=False, redraw=False)
-
-    def draw_facelabels(self, text=None, color=None):
-        """Draw labels for a selection of faces.
-
-        Parameters
-        ----------
-        text : dict, optional
-            A dictionary of face labels as face-text pairs.
-            The default value is ``None``, in which case every face will be labelled with its key.
-        color : tuple or dict of tuple, optional
-            The color sepcification of the labels.
-            The default color is the same as the default face color.
-
-        Returns
-        -------
-        list
-            The GUIDs of the created Rhino objects.
-
-        """
-        if not text or text == 'key':
-            face_text = {face: str(face) for face in self.mesh.faces()}
-        elif text == 'index':
-            face_text = {face: str(index) for index, face in enumerate(self.mesh.faces())}
-        elif isinstance(text, dict):
-            face_text = text
-        else:
-            raise NotImplementedError
-        vertex_xyz = self.vertex_xyz
-        face_color = colordict(color, face_text.keys(), default=self.color_faces)
-        labels = []
-        for face in face_text:
-            labels.append({
-                'pos': centroid_points([vertex_xyz[vertex] for vertex in self.mesh.face_vertices(face)]),
-                'name': "{}.facelabel.{}".format(self.mesh.name, face),
-                'color': face_color[face],
-                'text': face_text[face]})
+                'text': vertex_text[vertex]
+            })
         return compas_rhino.draw_labels(labels, layer=self.layer, clear=False, redraw=False)
 
     def draw_edgelabels(self, text=None, color=None):
@@ -435,7 +423,7 @@ class MeshArtist(BaseArtist):
             A dictionary of edge labels as edge-text pairs.
             The default value is ``None``, in which case every edge will be labelled with its key.
         color : tuple or dict of tuple, optional
-            The color sepcification of the labels.
+            The color specification of the labels.
             The default color is the same as the default color for edges.
 
         Returns
@@ -445,18 +433,57 @@ class MeshArtist(BaseArtist):
 
         """
         if text is None:
-            edge_text = {(u, v): "{}-{}".format(u, v) for u, v in self.mesh.edges()}
+            edge_text = {(u, v): "{}-{}".format(u, v) for u, v in self.edges}
         elif isinstance(text, dict):
             edge_text = text
         else:
             raise NotImplementedError
         vertex_xyz = self.vertex_xyz
-        edge_color = colordict(color, edge_text.keys(), default=self.color_edges)
+        edge_color = colordict(color, edge_text.keys(), default=self.default_edgecolor)
         labels = []
         for edge in edge_text:
             labels.append({
                 'pos': centroid_points([vertex_xyz[edge[0]], vertex_xyz[edge[1]]]),
                 'name': "{}.edgelabel.{}-{}".format(self.mesh.name, *edge),
                 'color': edge_color[edge],
-                'text': edge_text[edge]})
+                'text': edge_text[edge]
+            })
+        return compas_rhino.draw_labels(labels, layer=self.layer, clear=False, redraw=False)
+
+    def draw_facelabels(self, text=None, color=None):
+        """Draw labels for a selection of faces.
+
+        Parameters
+        ----------
+        text : dict, optional
+            A dictionary of face labels as face-text pairs.
+            The default value is ``None``, in which case every face will be labelled with its key.
+        color : tuple or dict of tuple, optional
+            The color specification of the labels.
+            The default color is the same as the default face color.
+
+        Returns
+        -------
+        list
+            The GUIDs of the created Rhino objects.
+
+        """
+        if not text or text == 'key':
+            face_text = {face: str(face) for face in self.faces}
+        elif text == 'index':
+            face_text = {face: str(index) for index, face in enumerate(self.faces)}
+        elif isinstance(text, dict):
+            face_text = text
+        else:
+            raise NotImplementedError
+        vertex_xyz = self.vertex_xyz
+        face_color = colordict(color, face_text.keys(), default=self.default_facecolor)
+        labels = []
+        for face in face_text:
+            labels.append({
+                'pos': centroid_points([vertex_xyz[vertex] for vertex in self.mesh.face_vertices(face)]),
+                'name': "{}.facelabel.{}".format(self.mesh.name, face),
+                'color': face_color[face],
+                'text': face_text[face]
+            })
         return compas_rhino.draw_labels(labels, layer=self.layer, clear=False, redraw=False)
